package org.fax4j.util;

import java.io.IOException;
import org.fax4j.FaxException;
import org.fax4j.common.Logger;
import org.fax4j.spi.FaxClientSpi;
import org.fax4j.util.ProcessExecutorHelper.ProcessOutput;

/**
 * The process executer enables to execute native processes.<br>
 * This class provides only partial implementation of the executer.<br>
 * Implementing classes must be statless and thread safe.
 * 
 * @author 	Sagie Gur-Ari
 * @version 1.02
 * @since	0.40.1
 */
public abstract class AbstractProcessExecutor implements ProcessExecutor
{
	/**
	 * This is the class constructor.
	 */
	public AbstractProcessExecutor()
	{
		super();
	}

	/**
	 * This function executes the given command and returns the process output.
	 * 
	 * @param	faxClientSpi
	 * 			The fax client SPI
	 * @param	command
	 * 			The command to execute
	 * @return	The process output
	 * @throws	IOException
	 * 			Any IO exception
	 * @throws	InterruptedException
	 * 			If thread interrupted during waitFor for the process
	 */
	public ProcessOutput executeProcess(FaxClientSpi faxClientSpi,String command) throws IOException,InterruptedException
	{
		//validate command provided
		if((command==null)||(command.length()==0))
		{
			throw new FaxException("Command not provided.");
		}

		//trim command and validate
		String updatedCommand=command.trim();
		if(updatedCommand.length()==0)
		{
			throw new FaxException("Command not provided.");
		}

		//validate fax client SPI provided
		if(faxClientSpi==null)
		{
			throw new FaxException("Fax client SPI not provided.");
		}

		//get logger
		Logger logger=faxClientSpi.getLogger();
		
		if(logger!=null)
		{
			logger.logDebug(new Object[]{"Invoking command: ",updatedCommand},null);
		}

		//execute process
		ProcessOutput processOutput=this.executeProcessImpl(faxClientSpi,updatedCommand);

		if(logger!=null)
		{
			int exitCode=processOutput.getExitCode();
			String outputText=processOutput.getOutputText();
			String errorText=processOutput.getErrorText();

			logger.logDebug(new Object[]{"Invoked command: ",updatedCommand," Exit Code: ",String.valueOf(exitCode),Logger.SYSTEM_EOL,"Output Text:",Logger.SYSTEM_EOL,outputText,Logger.SYSTEM_EOL,"Error Text:",Logger.SYSTEM_EOL,errorText},null);
		}
		
		return processOutput;
	}

	/**
	 * This function executes the given command and returns the process output.
	 * 
	 * @param	faxClientSpi
	 * 			The fax client SPI
	 * @param	command
	 * 			The command to execute
	 * @return	The process output
	 * @throws	IOException
	 * 			Any IO exception
	 * @throws	InterruptedException
	 * 			If thread interrupted during waitFor for the process
	 */
	protected abstract ProcessOutput executeProcessImpl(FaxClientSpi faxClientSpi,String command) throws IOException,InterruptedException;
}